/**
 *  Copyright 2009-2010 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/mirror.hpp>
#include <mirror/stream.hpp>
#include <mirror/factory.hpp>
#include <mirror/using_directive.hpp>
#include <mirror/utils/mijson_factory.hpp>

#include <cassert>
#include <iostream>
#include <stdexcept>

#include "./test.hpp"

MIRROR_NAMESPACE_BEGIN

MIRROR_NAMESPACE_END

int main(void)
{
	try
	{
		using namespace mirror;
		MIRROR_USING_NAMESPACE(test);
		//
		const char json[] =
			"["
			"	{"
			"		'title': 'dr', "
			"		'first_name': '\\u0414\\u043C\\u0438\\u0442\\u0440\\u0438\\u0439', "
			"		'middle_name': '\\u0418\\u0432\\u0430\\u043D\\u043E\\u0432\\u0438\\u0446', "
			"		'family_name': '\\u041C\\u0435\\u043D\\u0414\\u0435\\u043B\\u0435\\u0435\\u0432', "
			"		'birth_date': '1834-02-08', "
			"		'sex': 'male', "
			"		'weight': 75, "
			"		'height': 175"
			"	},"
			"	{"
			"		'title': 'mr', "
			"		'first_name': 'Johnny',"
			"		'middle_name': 'B.\\r\\n\\b\\t\\f\\\"\\'',"
			"		'family_name': 'Goode',"
			"		'birth_date': '1910-01-01',"
			"		'sex': 'male', "
			"		'weight': 70.4,"
			"		'height': 1.75e2"
			"	},"
			"	{"
			"		'title': 'mr', "
			"		'first_name': 'John',"
			"		'family_name': 'Doe',"
			"		'birth_date': '1955-05-10',"
			"		'sex': 'male', "
			"		'weight': 75.9,"
			"		'height': 182"
			"	}"
			"]";
		//
		mijson_factory_input in;
		mijson_factory_maker::factory<test::person>::type f = in.data();
		//
		mijson_fragment<const char*> fragment(
			json,
			json + sizeof(json) - 1
		);
		auto	i = fragment.elements().begin(),
			e = fragment.elements().end();
		while(i != e)
		{
			in.set(*i);
			test::person p(f());
			std::cout << stream::to_simple_struct::from(
				p,[](std::ostream& out){ out << "p"; }
			) << std::endl;
			++i;
		}
	}
	catch(std::exception const& error)
	{
		std::cerr << "Error: " << error.what() << std::endl;
	}
	//
	return 0;
}
