/**
 *  Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/ct_string.hpp>
#include <iostream>
#include <cassert>

template <typename S>
void test_str(void)
{
	using namespace mirror::cts;
	std::cout << c_str<S>() << std::endl;
	std::cout << (empty<S>::value ? "empty" : "non-empty") << std::endl;
	std::cout << length<S>::value << std::endl;
	std::cout << "'" << at_c<S, 0>::value << "'" << std::endl;
	std::cout << std::endl;
}

template <typename S1, typename S2>
void test_strs(void)
{
	using namespace mirror::cts;
	std::cout << "'" << c_str<S1>() << "', '" << c_str<S2>() << "'" << std::endl;
	std::cout << (equal<S1, S2>::value ? "equal" : "not equal") << std::endl;
	std::cout <<
		c_str<S1>() << " " <<
		(starts_with<S1, S2>::value ? "starts with" : "doesn't start with") <<
		" " << c_str<S2>() <<
		std::endl;
	std::cout <<
		c_str<S1>() << " " <<
		(ends_with<S1, S2>::value ? "ends with" : "doesn't end with") <<
		" " << c_str<S2>() <<
		std::endl;
	std::cout <<
		c_str<S1>() << " " <<
		(contains<S1, S2>::value ? "contains" : "doesn't contain") <<
		" " << c_str<S2>() <<
		std::endl;
	std::cout << std::endl;
}

int main(void)
{
	using namespace mirror::cts;
	typedef string<> e;
	typedef string<'o', 'b', 'j', 'e', 'c', 't'> s;
	typedef string<'_', '_'> u;
	typedef std::integral_constant<int, 9> n;
	//
	test_str<e>();
	test_str<concat<e,e,e,e,e>>();
	test_str<s>();
	test_str<concat<e,s,e,s,e>>();
	test_str<head<concat<e,s,e,s,e>, n>>();
	test_str<tail<concat<e,s,e,s,e>, n>>();
	test_str<concat<s,e,s,e,s>>();
	test_str<head_c<concat<s,e,s,e,s>, 13>>();
	test_str<concat<s>>();
	test_str<concat<s,s>>();
	test_str<concat<s,s,s>>();
	test_str<concat<s,s,s,s>>();
	test_str<concat<s,s,s,s,s>>();
	test_str<concat<s,s,s,s,s,s>>();
	test_str<concat<s,s,s,s,s,s,s>>();
	test_str<concat<s,s,s,s,s,s,s,s>>();
	test_str<concat<s,s,s,s,s,s,s,s,s>>();
	test_str<concat<s,s,s,s,s,s,s,s,s,s>>();
	test_str<concat<s,s,s,s,s,s,s,s,s,s,s>>();
	test_str<head<concat<s,s,s,s,s>, n>>();
	test_str<head<head_c<concat<s,s,s,s,s>, 13>, n>>();
	test_str<head_c<head<concat<s,s,s,s,s>, n>, 5>>();
	test_str<u>();
	test_str<head<u, n>>();
	test_str<tail<u, n>>();
	test_str<tail_c<tail<concat<s,s,s,s,s>, n>, 5>>();
	test_str<slice_c<concat<s,s,s,s,s>, 5, 10>>();
	test_str<skip_front_c<concat<s,s,s,s,s>, 5>>();
	test_str<skip_front_c<concat<s,s,s,s,s>, 27>>();
	test_str<find<concat<s,s,s,s,s>, tail_c<s, 3>>>();
	//
	test_strs<e, e>();
	test_strs<s, e>();
	test_strs<e, s>();
	test_strs<s, s>();
	test_strs<s, concat<s, e>>();
	test_strs<s, concat<s, s>>();
	test_strs<concat<s, s>, s>();
	test_strs<concat<e, s>, s>();
	test_strs<concat<s, u>, s>();
	test_strs<concat<u, s>, s>();
	test_strs<s, concat<s, u>>();
	test_strs<s, concat<u, s>>();
	//
	return 0;
}

