/**
 * @file mirror/utils/some_type.hpp
 * @brief A type erasure similar to boost any
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef MIRROR_UTILS_SOME_TYPE_1113061708_HPP
#define MIRROR_UTILS_SOME_TYPE_1113061708_HPP

#include <mirror/config.hpp>
#include <boost/any.hpp>
#include <type_traits>

MIRROR_NAMESPACE_BEGIN

class some
{
private:
	boost::any val_or_ptr;
	bool const_;
public:
	some(void) = default;

	template <typename T>
	some(T val, std::false_type = std::false_type())
	 : val_or_ptr(val)
	 , const_(std::is_const<T>::value)
	{ }

	template <typename T>
	some(T& val, std::true_type by_ptr)
	 : val_or_ptr(&const_cast<typename std::remove_cv<T>::type&>(val))
	 , const_(false)
	{ }

	template <typename T>
	some(const T& val, std::true_type by_ptr)
	 : val_or_ptr(&const_cast<typename std::remove_cv<T>::type&>(val))
	 , const_(true)
	{ }

	inline auto empty(void) const ->
	decltype(val_or_ptr.empty())
	{
		return val_or_ptr.empty();
	}

	inline auto none(void) const ->
	decltype(val_or_ptr.empty())
	{
		return val_or_ptr.empty();
	}

	template <typename T>
	inline bool is(void) const
	{
		return val_or_ptr.type() == typeid(T*) ||
			val_or_ptr.type() == typeid(T);
	}

	template <typename T>
	friend typename std::remove_volatile<
		typename std::remove_reference<T>::type
	>::type& some_cast(some& s)
	{
		assert(!s.val_or_ptr.empty());
		typedef typename std::remove_volatile<
			typename std::remove_reference<T>::type
		>::type type;
		typedef typename std::remove_const<type>::type type_;
		bool const_ok = !s.const_ || std::is_const<type>::value;
		if(s.val_or_ptr.type() == typeid(type_*) && const_ok)
			return *boost::any_cast<type_*>(s.val_or_ptr);
		else return boost::any_cast<type&>(s.val_or_ptr);
	}

	template <typename T>
	friend const typename std::remove_cv<
		typename std::remove_reference<T>::type
	>::type& some_cast(const some& s)
	{
		assert(!s.val_or_ptr.empty());
		typedef typename std::remove_cv<
			typename std::remove_reference<T>::type
		>::type type;
		if(s.val_or_ptr.type() == typeid(type*))
			return *boost::any_cast<type*>(s.val_or_ptr);
		else return boost::any_cast<const type&>(s.val_or_ptr);
	}

	template <typename T>
	inline auto as(void) const ->
	decltype(some_cast<const T>(*((const some*)nullptr)))
	{
		return some_cast<const T>(*this);
	}
};

template <typename T>
some some_ref(T& ref)
{
	return some(ref, std::true_type());
}

MIRROR_NAMESPACE_END

#endif //include guard

